jQuery(function($) {

	var controlled = false;

	$( document.body ).on( 'keyup keydown', function( e ) {
		controlled = e.ctrlKey || e.metaKey;
	});

	const recalculate_orders = function() {
		var $orders = $('[name^="per_product_order"]');
		$orders.each( function( index, el ) {
			$( el ).val( index );
		} );
	};

	const handle_pp_change = function( elem ) {
		const pp_parent_input = jQuery( elem );
		if ( pp_parent_input.is(':checked') ) {
			pp_parent_input.closest( 'div' ).find( '.per_product_shipping_rules' ).show();
		} else {
			pp_parent_input.closest( 'div' ).find( '.per_product_shipping_rules' ).hide();
		}
	};

	const handle_variable_virtual_change = function( elem ) {
		const current_elem   = jQuery( elem );
		const enable_pp_flag = current_elem.closest('.woocommerce_variation').find( '.enable_per_product_shipping' ).parent( 'label' );
		const variation_pp_rules = current_elem.closest( '.woocommerce_variation' ).find( '.per_product_shipping_rules' );

		if ( current_elem.is(':checked') ) {
			enable_pp_flag.hide();
			variation_pp_rules.hide();
		} else {
			enable_pp_flag.show();
			variation_pp_rules.show();
		}
	};

	const handle_enable_pp_change = function( elem ) {
		const current_elem = jQuery( elem );
		const variation_pp_rules = current_elem.closest( '.woocommerce_variation' ).find( '.per_product_shipping_rules' );

		if ( current_elem.is( ':checked' ) ) {
			variation_pp_rules.show();
		} else {
			variation_pp_rules.hide();
		}
	};

	$('body')
		.on( 'init_shipping_per_product', function() {
			handle_pp_change( 'input#_per_product_shipping' );

			jQuery( 'input.variable_is_virtual' ).each(
				function() {
					handle_variable_virtual_change( this );
				}
			);

			jQuery( '.enable_per_product_shipping' ).each(
				function() {
					handle_enable_pp_change( this );
				}
			);

			$('body').trigger( 'init_shipping_per_product_sortable' );
		})
		.on( 'wcspp_add_error_tip', function( e, element, error_type ) {
			var offset = element.position();

			if ( element.parent().find( '.wc_error_tip' ).length === 0 ) {
				element.after( '<div class="wc_error_tip ' + error_type + '">' + wc_shipping_per_product_errors[error_type] + '</div>' );
				element.parent().find( '.wc_error_tip' )
					.css( 'left', offset.left + element.width() - ( element.width() / 2 ) - ( $( '.wc_error_tip' ).width() / 2 ) )
					.css( 'top', offset.top + element.height() )
					.fadeIn( '100' );
			}
		})

		.on( 'wcspp_remove_error_tip', function( e, element, error_type ) {
			element.parent().find( '.wc_error_tip.' + error_type ).fadeOut( '100', function() { $( this ).remove(); } );
		})

		.on( 'blur', '.wcspp_country_validation', function() {
			var element = jQuery( this );
			var tip = element.parent().find( '.wc_error_tip' );

			if ( 0 < tip.length ) {
				element.val( '' );
			}
		})

		.on( 'keyup', '.wcspp_country_validation', function() {
			var error, value, newvalue;

			var value     = $( this ).val();
			var has_error = false;

			if ( $( this ).is( '.wcspp_country_validation' ) ) {
				error = 'i18n_incorrect_country_code';

				if ( 'undefined' === typeof wc_shipping_per_product_countries[ value ] && 2 === value.length ) {
					has_error = true;
				}
			}

			if ( has_error ) {
				$( document.body ).triggerHandler( 'wcspp_add_error_tip', [ $( this ), error ] );
			} else {
				$( document.body ).triggerHandler( 'wcspp_remove_error_tip', [ $( this ), error ] );
			}
		})

		.on( 'init_shipping_per_product_sortable', function() {
			$('.per_product_shipping_rules tbody').sortable({
				items:'tr',
				cursor:'move',
				axis:'y',
				scrollSensitivity:40,
				forcePlaceholderSize: true,
				helper: 'clone',
				opacity: 0.65,
				placeholder: 'wc-metabox-sortable-placeholder',
				start:function(event,ui){
					ui.item.css('background-color','#f6f6f6');
				},
				stop:function(event,ui){
					recalculate_orders();
					ui.item.removeAttr('style');
				}
			});
		})
		.on( 'change', 'input#_per_product_shipping', function() {
			handle_pp_change( this );
		})
		.on( 'change', 'input.variable_is_virtual', function() {
			handle_variable_virtual_change( this );
		});

	$('.woocommerce_variations')
		.on( 'change', '.enable_per_product_shipping', function() {
			handle_enable_pp_change( this );
		} )
		.on( 'woocommerce_variations_added woocommerce_variations_loaded', function() {
			$('body').trigger( 'init_shipping_per_product' );
		} );

	$( '#woocommerce-product-data' ).on( 'woocommerce_variations_added woocommerce_variations_loaded', function() {
		$('body').trigger( 'init_shipping_per_product' );
	} );

	$('#woocommerce-product-data')
		.on( 'focus', '.per_product_shipping_rules input', function() {
			if ( ! controlled ) {
				$('.per_product_shipping_rules tr').removeClass('current');
			}
			$(this).closest('tr').addClass('current');
		} )
		.on( 'click', '.per_product_shipping_rules input', function() {
			$(this).focus();
		  	return true;
		} )
		.on( 'click', '.per_product_shipping_rules .remove', function() {
			var $tbody = $(this).closest('.per_product_shipping_rules').find('tbody');
			if ( $tbody.find('tr.current').length ) {
				$tbody.find('tr.current').find('input').val('');
				$tbody.find('tr.current').hide();
			} else {
				alert( wc_shipping_per_product_params.i18n_no_row_selected );
			}
			return false;
		} )
		.on( 'click', '.per_product_shipping_rules .insert', function() {
			const cost_placeholder = '0{sep}00'.replace( '{sep}', wc_shipping_per_product_params.decimal_separator );
			var $tbody = $(this).closest('.per_product_shipping_rules').find('tbody');
			var postid = $(this).data('postid');
			var code = '<tr>\
				<td class="sort">&nbsp;<input type="hidden" value="" name="per_product_order[' + postid + '][new][]" /></td>\
				<td class="country"><input type="text" class="wcspp_country_validation" value="" maxlength="2" placeholder="*" name="per_product_country[' + postid + '][new][]" /></td>\
				<td class="state"><input type="text" value="" maxlength="7" placeholder="*" name="per_product_state[' + postid + '][new][]" /></td>\
				<td class="postcode"><input type="text" value="" placeholder="*" name="per_product_postcode[' + postid + '][new][]" /></td>\
				<td class="cost"><input type="text" class="wc_input_price input-text regular-input" value="' + cost_placeholder + '" placeholder="' + cost_placeholder + '" name="per_product_cost[' + postid + '][new][]" /></td>\
				<td class="item_cost"><input type="text" class="wc_input_price input-text regular-input" value="' + cost_placeholder + '" placeholder="' + cost_placeholder + '" name="per_product_item_cost[' + postid + '][new][]" /></td>\
			</tr>';

			if ( $tbody.find('tr.current').length ) {
				$tbody.find('tr.current').last().after( code );
			} else {
				$tbody.append( code );
			}
			recalculate_orders();
			return false;
		} )
		.on( 'click', '.per_product_shipping_rules .export', function( e ) {
			e.preventDefault();

			var export_btn = $( this ),
				post_id = $( this ).data( 'postid' );

			// Disable the export button while the CSV download is generated
			export_btn.attr( 'disabled', true );

			$.ajax( {
				url: ajaxurl,
				method: 'post',
				data: {
					action: 'wc_shipping_per_product_export_rules',
					product_id: post_id,
					security: $( '#_export_nonce' ).val(),
				},
				success: function( response ) {
					if ( response.success ) {
						const cost_placeholder = '"0{sep}00"'.replace( '{sep}', wc_shipping_per_product_params.decimal_separator );

						// Set the CSV header row
						var csv_string = wc_shipping_per_product_params.i18n_product_id + "," +
						                 wc_shipping_per_product_params.i18n_country_code + "," +
						                 wc_shipping_per_product_params.i18n_state + "," +
						                 wc_shipping_per_product_params.i18n_postcode + "," +
						                 wc_shipping_per_product_params.i18n_cost + "," +
						                 wc_shipping_per_product_params.i18n_item_cost + "\n";

						// Loop through each returned row and add to our CSV
						$.each( response.rules, function( index, rule ) {
							var columns, row;

							columns = [
								rule.product_id,
								rule.rule_country || '*',
								rule.rule_state || '*',
								rule.rule_postcode || '*',
								'"' + rule.rule_cost + '"' || cost_placeholder,
								'"' + rule.rule_item_cost + '"' || cost_placeholder,
							];

							row = columns.join();

							csv_string = csv_string + row + "\n";
						} );

						// Create a download link
						var download_link = document.createElement( 'a' );

						// Set the download link attributes
						download_link.href = URL.createObjectURL( new Blob( [csv_string] ) );
						download_link.download = 'per-product-rates-' + post_id + '.csv';

						// Download the CSV
						document.body.appendChild( download_link );
						download_link.click();
						document.body.removeChild( download_link );

						// Enable the export button again
						export_btn.attr( 'disabled', false );

						return true;
					}
				}
			} );
		} );

	$('body').trigger( 'init_shipping_per_product' );
});
